package org.processmining.logdecomposer.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.classification.XEventAndClassifier;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventLifeTransClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.log.dialogs.ClassifierPanel;
import org.processmining.logdecomposer.filters.DecompositionFilter;
import org.processmining.logdecomposer.filters.DecompositionFilterManager;
import org.processmining.logdecomposer.parameters.DecomposeEventLogUsingActivityClusterArrayParameters;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class DecomposeEventLogUsingActivityClusterArrayDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7737217369595500742L;

	public DecomposeEventLogUsingActivityClusterArrayDialog(XLog eventLog,
			final DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL, TableLayoutConstants.FILL, 30, 30 } };
		setLayout(new TableLayout(size));
		List<XEventClassifier> classifiers = new ArrayList<XEventClassifier>(eventLog.getClassifiers());
		if (classifiers.isEmpty()) {
			classifiers.add(new XEventAndClassifier(new XEventNameClassifier(), new XEventLifeTransClassifier()));
		}
		add(new ClassifierPanel(classifiers, parameters), "0, 0");

		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for (DecompositionFilter filter : DecompositionFilterManager.getInstance().getMiners()) {
			listModel.addElement(filter.getName());
		}
		final ProMList<String> list = new ProMList<String>("Select filter", listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		final String defaultFilter = DecompositionFilterManager.getInstance().getMiner(null).getName();
		list.setSelection(defaultFilter);
		list.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				List<String> selected = list.getSelectedValuesList();
				if (selected.size() == 1) {
					parameters.setFilter(selected.get(0));
				} else {
					/*
					 * Nothing selected. Revert to selection of default classifier.
					 */
					list.setSelection(defaultFilter);
					parameters.setFilter(defaultFilter);
				}
			}
		});
		list.setPreferredSize(new Dimension(100, 100));
		add(list, "0, 1");

		final JCheckBox emptyBox = SlickerFactory.instance().createCheckBox("Remove empty traces", false);
		emptyBox.setSelected(parameters.isRemoveEmptyTraces());
		emptyBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				parameters.setRemoveEmptyTraces(emptyBox.isSelected());
			}

		});
		emptyBox.setOpaque(false);
		add(emptyBox, "0, 2");

		final JCheckBox artificialBox = SlickerFactory.instance().createCheckBox("Add artifical Start and End events", false);
		artificialBox.setSelected(parameters.isAddStartEndEvents());
		artificialBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				parameters.setAddStartEndEvents(artificialBox.isSelected());
			}

		});
		artificialBox.setOpaque(false);
		add(artificialBox, "0, 3");
	}
}
