package org.processmining.logdecomposer.filters;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.log.parameters.ClassifierParameter;
import org.processmining.log.utils.XUtils;

public class DecompositionFilterParameters extends PluginParametersImpl implements ClassifierParameter {

	private XEventClassifier classifier;
	private boolean removeEmptyTraces;
	private boolean addStartEndEvents;
	private String filter;

	public DecompositionFilterParameters(XLog log) {
		super();
		setClassifier(XUtils.getDefaultClassifier(log));
		setRemoveEmptyTraces(false);
		setAddStartEndEvents(true);
	}
	
	public DecompositionFilterParameters(DecompositionFilterParameters parameters) {
		super(parameters);
		setClassifier(parameters.getClassifier());
		setRemoveEmptyTraces(parameters.isRemoveEmptyTraces());
		setAddStartEndEvents(parameters.isAddStartEndEvents());
	}
	
	public void setClassifier(XEventClassifier classifier) {
		this.classifier = classifier;
	}
	
	public XEventClassifier getClassifier() {
		return classifier;
	}
	
	public boolean equals(Object object) {
		if (object instanceof DecompositionFilterParameters) {
			DecompositionFilterParameters parameters = (DecompositionFilterParameters) object;
			return super.equals(parameters) && getClassifier().equals(parameters.getClassifier())
					&& isRemoveEmptyTraces() == parameters.isRemoveEmptyTraces()
					&& isAddStartEndEvents() == parameters.isAddStartEndEvents();
		}
		return false;
	}

	public void setRemoveEmptyTraces(Boolean removeEmptyTraces) {
		this.removeEmptyTraces = removeEmptyTraces;
	}

	/**
	 * deprecated Use isRemoveEmptyTraces() instead.
	 */
	@Deprecated
	public Boolean getRemoveEmptyTraces() {
		return removeEmptyTraces;
	}

	public Boolean isRemoveEmptyTraces() {
		return removeEmptyTraces;
	}

	public void setFilter(String filter) {
		this.filter = filter;
	}

	public String getFilter() {
		return filter;
	}

	public boolean isAddStartEndEvents() {
		return addStartEndEvents;
	}

	public void setAddStartEndEvents(boolean addStartEndEvents) {
		this.addStartEndEvents = addStartEndEvents;
	}

}
