package org.processmining.logdecomposer.filters.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.basicutils.utils.StringConstants;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.models.EventLogArray;
import org.processmining.log.models.impl.EventLogArrayFactory;
import org.processmining.logdecomposer.classifiers.StartEndClassifier;
import org.processmining.logdecomposer.extensions.XClassExtension;
import org.processmining.logdecomposer.filters.DecompositionFilter;
import org.processmining.logdecomposer.parameters.DecomposeEventLogUsingActivityClusterArrayParameters;

public class DecompositionOutFilter implements DecompositionFilter {

	public final static String NAME = "Filter Out";
	
	public EventLogArray apply(PluginContext context, XLog eventLog, ActivityClusterArray clusters,
			DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		EventLogArray eventLogs = EventLogArrayFactory.createEventLogArray();
		eventLogs.init();
		XEventClassifier artificialClassifier = new StartEndClassifier(parameters.getClassifier());

		for (Set<XEventClass> activities : clusters.getClusters()) {
			Set<String> labels = new HashSet<String>();
			for (XEventClass activity : activities) {
				labels.add(activity.getId());
			}
			XLog subLog = filterLog(eventLog, parameters.getClassifier(), labels, parameters.isRemoveEmptyTraces(), parameters.isAddStartEndEvents());
			if (parameters.isAddStartEndEvents()) {
				subLog.getClassifiers().add(0, artificialClassifier);
			}
			eventLogs.addLog(subLog);
		}
		return eventLogs;
	}

	private XLog filterLog(XLog eventLog, XEventClassifier classifier, Collection<String> labels,
			boolean removeEmptyTraces, boolean isAddStartEndEvents) {
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog filteredLog = (XLog) eventLog.clone();
		Iterator<XTrace> itTrace = filteredLog.iterator();
		while (itTrace.hasNext()) {
			XTrace trace = itTrace.next();
			Iterator<XEvent> itEvent = trace.iterator();
			while (itEvent.hasNext()) {
				XEvent event = itEvent.next();
				String id = classifier.getClassIdentity(event);
				if (!labels.contains(id)) {
					itEvent.remove();
				}
			}
			if (isAddStartEndEvents) {
				XEvent startEvent = factory.createEvent();
				XClassExtension.instance().assignEventId(startEvent, StringConstants.START);
				trace.add(0, startEvent);
				XEvent endEvent = factory.createEvent();
				XClassExtension.instance().assignEventId(endEvent, StringConstants.END);
				trace.add(endEvent);
			}
			if (removeEmptyTraces && trace.isEmpty()) {
				itTrace.remove();
			}
		}
		return filteredLog;
	}

	public String getName() {
		return NAME;
	}

}
