package org.processmining.logdecomposer.filters.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.basicutils.utils.StringConstants;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.models.EventLogArray;
import org.processmining.log.models.impl.EventLogArrayFactory;
import org.processmining.logdecomposer.classifiers.StartEndClassifier;
import org.processmining.logdecomposer.extensions.XClassExtension;
import org.processmining.logdecomposer.filters.DecompositionFilter;
import org.processmining.logdecomposer.parameters.DecomposeEventLogUsingActivityClusterArrayParameters;
import org.processmining.plugins.log.logfilters.LogFilter;
import org.processmining.plugins.log.logfilters.LogFilterException;
import org.processmining.plugins.log.logfilters.XEventEditor;
import org.processmining.plugins.log.logfilters.XTraceEditor;

public class DecompositionRenameFilter implements DecompositionFilter {

	public final static String NAME = "Rename";
	private boolean lastDummy;
	
	public EventLogArray apply(PluginContext context, XLog eventLog, ActivityClusterArray clusters,
			DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		EventLogArray eventLogs = EventLogArrayFactory.createEventLogArray();
		eventLogs.init();
		XEventClassifier artificialClassifier = new StartEndClassifier(parameters.getClassifier());

		for (Set<XEventClass> activities : clusters.getClusters()) {
			Set<String> labels = new HashSet<String>();
			for (XEventClass activity : activities) {
				labels.add(activity.getId());
			}
			lastDummy = false;
			XLog subLog = editLog(eventLog, parameters.getClassifier(), labels, parameters.isAddStartEndEvents());
			if (parameters.isAddStartEndEvents()) {
				subLog.getClassifiers().add(0, artificialClassifier);
			}
			eventLogs.addLog(subLog);
		}
		return eventLogs;
	}

	private XLog editLog(XLog eventLog, final XEventClassifier classifier, final Collection<String> labels, final boolean isAddStartEndEvents) {

		final XFactory factory = XFactoryRegistry.instance().currentDefault();
		XAttributeMap attributes = factory.createAttributeMap();
		for (XAttribute globalAttribute : eventLog.getGlobalEventAttributes()) {
			XAttribute attribute = (XAttribute) globalAttribute.clone();
			attributes.put(globalAttribute.getKey(), attribute);
		}
		final XEvent dummyEvent = factory.createEvent(attributes);

		try {
			return LogFilter.filter(eventLog, new XEventEditor() {

				public XEvent editEvent(XEvent event) {
					if (labels.contains(classifier.getClassIdentity(event))) {
						lastDummy = false;
						return event;
					}
					if (!lastDummy) {
						lastDummy = false;
						return dummyEvent;
					}
					return dummyEvent; //null;
				}
			}, new XTraceEditor() {

				public XTrace editTrace(XTrace trace) {
					lastDummy = false;
					if (isAddStartEndEvents) {
						XEvent startEvent = factory.createEvent();
						XClassExtension.instance().assignEventId(startEvent, StringConstants.START);
						trace.add(0, startEvent);
						XEvent endEvent = factory.createEvent();
						XClassExtension.instance().assignEventId(endEvent, StringConstants.END);
						trace.add(endEvent);
					}
					return trace;
				}

			});
		} catch (LogFilterException e) {
			e.printStackTrace();
		}
		return null;
	}

	public String getName() {
		return NAME;
	}

}
